@@ -4,12 +4,12 @@
    responses. */
 
 /*
- * Copyright (c) 2004,2007 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2017 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1996-2003 by Internet Software Consortium
  *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
  * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
  * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
@@ -23,14 +23,8 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   http://www.isc.org/
+ *   https://www.isc.org/
  *
- * This software has been written for Internet Systems Consortium
- * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
- * To learn more about Internet Systems Consortium, see
- * ``http://www.isc.org/''.  To learn more about Vixie Enterprises,
- * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
- * ``http://www.nominum.com''.
  */
 
 #include "dhcpd.h"
@@ -52,7 +46,7 @@ trace_type_t *trace_icmp_output;
 
 void icmp_startup (routep, handler)
 	int routep;
-	void (*handler) PROTO ((struct iaddr, u_int8_t *, int));
+	void (*handler) (struct iaddr, u_int8_t *, int);
 {
 	struct protoent *proto;
 	int protocol = 1;
@@ -288,17 +282,15 @@ void trace_icmp_input_stop (trace_type_t *ttype) { }
 
 void trace_icmp_output_input (trace_type_t *ttype, unsigned length, char *buf)
 {
-	struct icmp *icmp;
 	struct iaddr ia;
 
-	if (length != (sizeof (*icmp) + (sizeof ia))) {
+	if (length != (sizeof (struct icmp) + sizeof (ia))) {
 		log_error ("trace_icmp_output_input: data size mismatch %d:%d",
-			   length, (int)((sizeof (*icmp)) + (sizeof ia)));
+			   length, (int)(sizeof (struct icmp) + sizeof (ia)));
 		return;
 	}
 	ia.len = 4;
 	memcpy (ia.iabuf, buf, 4);
-	icmp = (struct icmp *)(buf + 1);
 
 	log_error ("trace_icmp_output_input: unsent ping to %s", piaddr (ia));
 }
