@@ -3,12 +3,12 @@
    Parser for /etc/resolv.conf file. */
 
 /*
- * Copyright (c) 2004-2006 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2017 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1996-2003 by Internet Software Consortium
  *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
  * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
  * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
@@ -24,19 +24,8 @@
  *   <info@isc.org>
  *   https://www.isc.org/
  *
- * This software has been written for Internet Systems Consortium
- * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
- * To learn more about Internet Systems Consortium, see
- * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
- * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
- * ``http://www.nominum.com''.
  */
 
-#ifndef lint
-static char copyright[] =
-"$Id: resolv.c,v 1.18.116.1 2009/07/23 21:43:34 sar Exp $ Copyright (c) 2004-2006 Internet Systems Consortium.  All rights reserved.\n";
-#endif /* not lint */
-
 #include "dhcpd.h"
 
 struct name_server *name_servers;
@@ -50,18 +39,19 @@ void read_resolv_conf (parse_time)
 	struct parse *cfile;
 	const char *val;
 	int token;
-	int declaration = 0;
 	struct name_server *sp, *sl, *ns;
 	struct domain_search_list *dp, *dl, *nd;
-	struct iaddr *iaddr;
+	isc_result_t status;
 
 	if ((file = open (path_resolv_conf, O_RDONLY)) < 0) {
 		log_error ("Can't open %s: %m", path_resolv_conf);
 		return;
 	}
 
-	cfile = (struct parse *)0;
-	new_parse (&cfile, file, (char *)0, 0, path_resolv_conf, 1);
+	cfile = NULL;
+	status = new_parse(&cfile, file, NULL, 0, path_resolv_conf, 1);
+	if (status != ISC_R_SUCCESS || cfile == NULL)
+		return;
 
 	do {
 		token = next_token (&val, (unsigned *)0, cfile);
@@ -93,7 +83,6 @@ void read_resolv_conf (parse_time)
 						(struct domain_search_list *)0;
 					*dp = nd;
 					nd -> domain = dn;
-					dn = (char *)0;
 				}
 				nd -> rcdate = parse_time;
 				token = peek_token (&val,
@@ -104,7 +93,7 @@ void read_resolv_conf (parse_time)
 					    "junk after domain declaration");
 				skip_to_semi (cfile);
 			}
-			token = next_token (&val, (unsigned *)0, cfile);
+			skip_token(&val, (unsigned *)0, cfile);
 		} else if (token == NAMESERVER) {
 			struct name_server *ns, **sp;
 			struct iaddr iaddr;
@@ -140,7 +129,7 @@ void read_resolv_conf (parse_time)
 		} else
 			skip_to_semi (cfile); /* Ignore what we don't grok. */
 	} while (1);
-	token = next_token (&val, (unsigned *)0, cfile);
+	skip_token(&val, (unsigned *)0, cfile);
 
 	/* Lose servers that are no longer in /etc/resolv.conf. */
 	sl = (struct name_server *)0;
@@ -154,7 +143,7 @@ void read_resolv_conf (parse_time)
 			/* We can't actually free the name server structure,
 			   because somebody might be hanging on to it.    If
 			   your /etc/resolv.conf file changes a lot, this
-			   could be a noticable memory leak. */
+			   could be a noticeable memory leak. */
 		} else
 			sl = sp;
 	}
@@ -179,7 +168,6 @@ void read_resolv_conf (parse_time)
 
 struct name_server *first_name_server ()
 {
-	FILE *rc;
 	static TIME rcdate;
 	struct stat st;
 
@@ -190,8 +178,6 @@ struct name_server *first_name_server ()
 			return (struct name_server *)0;
 		}
 		if (st.st_mtime > rcdate) {
-			char rcbuf [512];
-			char *s, *t, *u;
 			rcdate = cur_time + 1;
 			
 			read_resolv_conf (rcdate);
