@@ -3,12 +3,12 @@
    DHCP Server Daemon. */
 
 /*
- * Copyright (c) 2004-2016 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2018 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1996-2003 by Internet Software Consortium
  *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
  * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
  * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
@@ -27,7 +27,7 @@
  */
 
 static const char copyright[] =
-"Copyright 2004-2016 Internet Systems Consortium.";
+"Copyright 2004-2018 Internet Systems Consortium.";
 static const char arr [] = "All rights reserved.";
 static const char message [] = "Internet Systems Consortium DHCP Server";
 static const char url [] =
@@ -48,7 +48,7 @@ static const char url [] =
 #  include <unistd.h>
 #  include <pwd.h>
 /* get around the ISC declaration of group */
-#  define group real_group 
+#  define group real_group
 #    include <grp.h>
 #  undef group
 
@@ -57,6 +57,9 @@ uid_t set_uid = 0;
 gid_t set_gid = 0;
 #endif /* PARANOIA */
 
+struct class unknown_class;
+struct class known_class;
+
 struct iaddr server_identifier;
 int server_identifier_matched;
 
@@ -70,14 +73,26 @@ option server.ddns-hostname =						    \n\
 option server.ddns-domainname =	config-option domain-name;		    \n\
 option server.ddns-rev-domainname = \"in-addr.arpa.\";";
 
+/* Stores configured DDNS conflict detection flags */
+u_int16_t ddns_conflict_mask;
 #endif /* NSUPDATE */
+
 int ddns_update_style;
 int dont_use_fsync = 0; /* 0 = default, use fsync, 1 = don't use fsync */
 int server_id_check = 0; /* 0 = default, don't check server id, 1 = do check */
-int prefix_length_mode = PLM_EXACT;
+
+#ifdef DHCPv6
+int prefix_length_mode = PLM_PREFER;
+int do_release_on_roam = 0; /* 0 = default, do not release v6 leases on roam */
+#endif
+
+#ifdef EUI_64
+int persist_eui64 = 1; /* 1 = write EUI64 leases to disk, 0 = don't */
+#endif
 
 int authoring_byte_order = 0; /* 0 = not set */
 int lease_id_format = TOKEN_OCTAL; /* octal by default */
+u_int32_t abandon_lease_time = DEFAULT_ABANDON_LEASE_TIME;
 
 const char *path_dhcpd_conf = _PATH_DHCPD_CONF;
 const char *path_dhcpd_db = _PATH_DHCPD_DB;
@@ -134,6 +149,44 @@ static void omapi_listener_start (void *foo)
 
 #ifndef UNIT_TEST
 
+#define DHCPD_USAGE0 \
+"[-p <UDP port #>] [-f] [-d] [-q] [-t|-T]\n"
+
+#ifdef DHCPv6
+#ifdef DHCP4o6
+#define DHCPD_USAGE1 \
+"             [-4|-6] [-4o6 <port>]\n" \
+"             [-cf config-file] [-lf lease-file]\n"
+#else /* DHCP4o6 */
+#define DHCPD_USAGE1 \
+"             [-4|-6] [-cf config-file] [-lf lease-file]\n"
+#endif /* DHCP4o6 */
+#else /* !DHCPv6 */
+#define DHCPD_USAGE1 \
+"             [-cf config-file] [-lf lease-file]\n"
+#endif /* DHCPv6 */
+
+#if defined (PARANOIA)
+#define DHCPD_USAGEP \
+"             [-user user] [-group group] [-chroot dir]\n"
+#else
+#define DHCPD_USAGEP ""
+#endif /* PARANOIA */
+
+#if defined (TRACING)
+#define DHCPD_USAGET \
+"             [-tf trace-output-file]\n" \
+"             [-play trace-input-file]\n"
+#else
+#define DHCPD_USAGET ""
+#endif /* TRACING */
+
+#define DHCPD_USAGEC \
+"             [-pf pid-file] [--no-pid] [-s server]\n" \
+"             [if0 [...ifN]]"
+
+#define DHCPD_USAGEH "{--version|--help|-h}"
+
 /*!
  *
  * \brief Print the generic usage message
@@ -156,6 +209,7 @@ usage(const char *sfmt, const char *sarg) {
 	log_info("%s %s", message, PACKAGE_VERSION);
 	log_info(copyright);
 	log_info(arr);
+	log_info(url);
 
 	/* If desired print out the specific error message */
 #ifdef PRINT_SPECIFIC_CL_ERRORS
@@ -163,28 +217,15 @@ usage(const char *sfmt, const char *sarg) {
 		log_error(sfmt, sarg);
 #endif
 
-	log_fatal("Usage: %s [-p <UDP port #>] [-f] [-d] [-q] [-t|-T]\n"
-#ifdef DHCPv6
-#ifdef DHCP4o6
-		  "             [-4|-6] [-4o6 <port>]\n"
-		  "             [-cf config-file] [-lf lease-file]\n"
-#else /* DHCP4o6 */
-		  "             [-4|-6] [-cf config-file] [-lf lease-file]\n"
-#endif /* DHCP4o6 */
-#else /* !DHCPv6 */
-		  "             [-cf config-file] [-lf lease-file]\n"
-#endif /* DHCPv6 */
-#if defined (PARANOIA)
-		   /* meld into the following string */
-		  "             [-user user] [-group group] [-chroot dir]\n"
-#endif /* PARANOIA */
-#if defined (TRACING)
-		  "             [-tf trace-output-file]\n"
-		  "             [-play trace-input-file]\n"
-#endif /* TRACING */
-		  "             [-pf pid-file] [--no-pid] [-s server]\n"
-		  "             [if0 [...ifN]]",
-		  isc_file_basename(progname));
+	log_fatal("Usage: %s %s%s%s%s%s\n       %s %s",
+		  isc_file_basename(progname),
+		  DHCPD_USAGE0,
+		  DHCPD_USAGE1,
+		  DHCPD_USAGEP,
+		  DHCPD_USAGET,
+		  DHCPD_USAGEC,
+		  isc_file_basename(progname),
+		  DHCPD_USAGEH);
 }
 
 /* Note: If we add unit tests to test setup_chroot it will
@@ -207,7 +248,7 @@ static void setup_chroot (char *chroot_dir) {
 }
 #endif /* PARANOIA */
 
-int 
+int
 main(int argc, char **argv) {
 	int fd;
 	int i, status;
@@ -219,6 +260,7 @@ main(int argc, char **argv) {
 	char pbuf [20];
 #ifndef DEBUG
 	int daemon = 1;
+	int dfd[2] = { -1, -1 };
 #endif
 	int quiet = 0;
 	char *server = (char *)0;
@@ -229,9 +271,9 @@ main(int argc, char **argv) {
 	struct parse *parse;
 	int lose;
 #endif
-	int no_dhcpd_conf = 0;
-	int no_dhcpd_db = 0;
-	int no_dhcpd_pid = 0;
+	int have_dhcpd_conf = 0;
+	int have_dhcpd_db = 0;
+	int have_dhcpd_pid = 0;
 #ifdef DHCPv6
 	int local_family_set = 0;
 #ifdef DHCP4o6
@@ -268,6 +310,96 @@ main(int argc, char **argv) {
         else if (fd != -1)
                 close(fd);
 
+	/* Parse arguments changing daemon */
+	for (i = 1; i < argc; i++) {
+		if (!strcmp (argv [i], "-f")) {
+#ifndef DEBUG
+			daemon = 0;
+#endif
+		} else if (!strcmp (argv [i], "-d")) {
+#ifndef DEBUG
+			daemon = 0;
+#endif
+		} else if (!strcmp (argv [i], "-t")) {
+#ifndef DEBUG
+			daemon = 0;
+#endif
+		} else if (!strcmp (argv [i], "-T")) {
+#ifndef DEBUG
+			daemon = 0;
+#endif
+		} else if (!strcmp (argv [i], "--version")) {
+			const char vstring[] = "isc-dhcpd-";
+			IGNORE_RET(write(STDERR_FILENO, vstring,
+					 strlen(vstring)));
+			IGNORE_RET(write(STDERR_FILENO,
+					 PACKAGE_VERSION,
+					 strlen(PACKAGE_VERSION)));
+			IGNORE_RET(write(STDERR_FILENO, "\n", 1));
+			exit (0);
+		} else if (!strcmp(argv[i], "--help") ||
+			   !strcmp(argv[i], "-h")) {
+			const char *pname = isc_file_basename(progname);
+			IGNORE_RET(write(STDERR_FILENO, "Usage: ", 7));
+			IGNORE_RET(write(STDERR_FILENO, pname, strlen(pname)));
+			IGNORE_RET(write(STDERR_FILENO, " ", 1));
+			IGNORE_RET(write(STDERR_FILENO, DHCPD_USAGE0,
+					 strlen(DHCPD_USAGE0)));
+			IGNORE_RET(write(STDERR_FILENO, DHCPD_USAGE1,
+					 strlen(DHCPD_USAGE1)));
+#if defined (PARANOIA)
+			IGNORE_RET(write(STDERR_FILENO, DHCPD_USAGEP,
+					 strlen(DHCPD_USAGEP)));
+#endif
+#if defined (TRACING)
+			IGNORE_RET(write(STDERR_FILENO, DHCPD_USAGET,
+					 strlen(DHCPD_USAGET)));
+#endif
+			IGNORE_RET(write(STDERR_FILENO, DHCPD_USAGEC,
+					 strlen(DHCPD_USAGEC)));
+			IGNORE_RET(write(STDERR_FILENO, "\n", 1));
+			IGNORE_RET(write(STDERR_FILENO, "       ", 7));
+			IGNORE_RET(write(STDERR_FILENO, pname, strlen(pname)));
+			IGNORE_RET(write(STDERR_FILENO, " ", 1));
+			IGNORE_RET(write(STDERR_FILENO, DHCPD_USAGEH,
+					 strlen(DHCPD_USAGEH)));
+			IGNORE_RET(write(STDERR_FILENO, "\n", 1));
+			exit(0);
+#ifdef TRACING
+		} else if (!strcmp (argv [i], "-play")) {
+#ifndef DEBUG
+			daemon = 0;
+#endif
+#endif
+		}
+	}
+
+#ifndef DEBUG
+	/* When not forbidden prepare to become a daemon */
+	if (daemon) {
+		if (pipe(dfd) == -1)
+			log_fatal("Can't get pipe: %m");
+		if ((pid = fork ()) < 0)
+			log_fatal("Can't fork daemon: %m");
+		if (pid != 0) {
+			/* Parent: wait for the child to start */
+			int n;
+
+			(void) close(dfd[1]);
+			do {
+				char buf;
+
+				n = read(dfd[0], &buf, 1);
+				if (n == 1)
+					_exit((int)buf);
+			} while (n == -1 && errno == EINTR);
+			_exit(1);
+		}
+		/* Child */
+		(void) close(dfd[0]);
+	}
+#endif
+
 	/* Set up the isc and dns library managers */
 	status = dhcp_context_create(DHCP_CONTEXT_PRE_DB,
 				     NULL, NULL);
@@ -303,11 +435,11 @@ main(int argc, char **argv) {
 			       ntohs (local_port));
 		} else if (!strcmp (argv [i], "-f")) {
 #ifndef DEBUG
-			daemon = 0;
+			/* daemon = 0; */
 #endif
 		} else if (!strcmp (argv [i], "-d")) {
 #ifndef DEBUG
-			daemon = 0;
+			/* daemon = 0; */
 #endif
 			log_perror = -1;
 		} else if (!strcmp (argv [i], "-s")) {
@@ -332,30 +464,30 @@ main(int argc, char **argv) {
 			if (++i == argc)
 				usage(use_noarg, argv[i-1]);
 			path_dhcpd_conf = argv [i];
-			no_dhcpd_conf = 1;
+			have_dhcpd_conf = 1;
 		} else if (!strcmp (argv [i], "-lf")) {
 			if (++i == argc)
 				usage(use_noarg, argv[i-1]);
 			path_dhcpd_db = argv [i];
-			no_dhcpd_db = 1;
+			have_dhcpd_db = 1;
 		} else if (!strcmp (argv [i], "-pf")) {
 			if (++i == argc)
 				usage(use_noarg, argv[i-1]);
 			path_dhcpd_pid = argv [i];
-			no_dhcpd_pid = 1;
+			have_dhcpd_pid = 1;
 		} else if (!strcmp(argv[i], "--no-pid")) {
 			no_pid_file = ISC_TRUE;
                 } else if (!strcmp (argv [i], "-t")) {
 			/* test configurations only */
 #ifndef DEBUG
-			daemon = 0;
+			/* daemon = 0; */
 #endif
 			cftest = 1;
 			log_perror = -1;
                 } else if (!strcmp (argv [i], "-T")) {
 			/* test configurations and lease file only */
 #ifndef DEBUG
-			daemon = 0;
+			/* daemon = 0; */
 #endif
 			cftest = 1;
 			lftest = 1;
@@ -390,15 +522,6 @@ main(int argc, char **argv) {
 			dhcpv4_over_dhcpv6 = 1;
 #endif /* DHCP4o6 */
 #endif /* DHCPv6 */
-		} else if (!strcmp (argv [i], "--version")) {
-			const char vstring[] = "isc-dhcpd-";
-			IGNORE_RET(write(STDERR_FILENO, vstring,
-					 strlen(vstring)));
-			IGNORE_RET(write(STDERR_FILENO,
-					 PACKAGE_VERSION,
-					 strlen(PACKAGE_VERSION)));
-			IGNORE_RET(write(STDERR_FILENO, "\n", 1));
-			exit (0);
 #if defined (TRACING)
 		} else if (!strcmp (argv [i], "-tf")) {
 			if (++i == argc)
@@ -447,47 +570,44 @@ main(int argc, char **argv) {
 	}
 #endif /* DHCPv6 && DHCP4o6 */
 
-	if (!no_dhcpd_conf && (s = getenv ("PATH_DHCPD_CONF"))) {
+	if (!have_dhcpd_conf && (s = getenv ("PATH_DHCPD_CONF"))) {
 		path_dhcpd_conf = s;
 	}
 
 #ifdef DHCPv6
         if (local_family == AF_INET6) {
                 /* DHCPv6: override DHCPv4 lease and pid filenames */
-	        if (!no_dhcpd_db) {
+	        if (!have_dhcpd_db) {
                         if ((s = getenv ("PATH_DHCPD6_DB")))
 		                path_dhcpd_db = s;
                         else
 		                path_dhcpd_db = _PATH_DHCPD6_DB;
 	        }
-	        if (!no_dhcpd_pid) {
+	        if (!have_dhcpd_pid) {
                         if ((s = getenv ("PATH_DHCPD6_PID")))
 		                path_dhcpd_pid = s;
                         else
 		                path_dhcpd_pid = _PATH_DHCPD6_PID;
 	        }
         } else
-#else /* !DHCPv6 */
+#endif /* DHCPv6 */
         {
-	        if (!no_dhcpd_db && (s = getenv ("PATH_DHCPD_DB"))) {
+	        if (!have_dhcpd_db && (s = getenv ("PATH_DHCPD_DB"))) {
 		        path_dhcpd_db = s;
+			have_dhcpd_db = 1;
 	        }
-	        if (!no_dhcpd_pid && (s = getenv ("PATH_DHCPD_PID"))) {
+	        if (!have_dhcpd_pid && (s = getenv ("PATH_DHCPD_PID"))) {
 		        path_dhcpd_pid = s;
+			have_dhcpd_pid = 1;
 	        }
         }
-#endif /* DHCPv6 */
 
         /*
          * convert relative path names to absolute, for files that need
          * to be reopened after chdir() has been called
          */
-        if (path_dhcpd_db[0] != '/') {
-		const char *path = path_dhcpd_db;
-                path_dhcpd_db = realpath(path_dhcpd_db, NULL);
-                if (path_dhcpd_db == NULL)
-                        log_fatal("Failed to get realpath for %s: %s", path, 
-                                   strerror(errno));
+        if (have_dhcpd_db && path_dhcpd_db[0] != '/') {
+                path_dhcpd_db = absolute_path(path_dhcpd_db);
         }
 
 	if (!quiet) {
@@ -584,7 +704,7 @@ main(int argc, char **argv) {
 #endif
 		}
 	}
-  
+
   	if (local_family == AF_INET) {
 		remote_port = htons(ntohs(local_port) + 1);
 	} else {
@@ -672,18 +792,18 @@ main(int argc, char **argv) {
 
 #if defined (TRACING)
 	if (traceinfile) {
-	    if (!no_dhcpd_db) {
+	    if (!have_dhcpd_db) {
 		    log_error ("%s", "");
 		    log_error ("** You must specify a lease file with -lf.");
 		    log_error ("   Dhcpd will not overwrite your default");
 		    log_fatal ("   lease file when playing back a trace. **");
-	    }		
+	    }
 	    trace_file_replay (traceinfile);
 
 #if defined (DEBUG_MEMORY_LEAKAGE) && \
                 defined (DEBUG_MEMORY_LEAKAGE_ON_EXIT)
             free_everything ();
-            omapi_print_dmalloc_usage_by_caller (); 
+            omapi_print_dmalloc_usage_by_caller ();
 #endif
 
 	    exit (0);
@@ -709,6 +829,10 @@ main(int argc, char **argv) {
 
 	postconf_initialization (quiet);
 
+#if defined (FAILOVER_PROTOCOL)
+	dhcp_failover_sanity_check();
+#endif
+
 #if defined(DHCPv6) && defined(DHCP4o6)
 	if (dhcpv4_over_dhcpv6) {
 		if ((local_family == AF_INET) && (interfaces != NULL))
@@ -728,8 +852,9 @@ main(int argc, char **argv) {
 #endif
 
         /* test option should cause an early exit */
- 	if (cftest && !lftest) 
+	if (cftest && !lftest) {
  		exit(0);
+	}
 
 	/*
 	 * First part of dealing with pid files.  Check to see if
@@ -789,8 +914,8 @@ main(int argc, char **argv) {
 	 * Remove addresses from our pools that we should not issue
 	 * to clients.
 	 *
-	 * We currently have no support for this in IPv4. It is not 
-	 * as important in IPv4, as making pools with ranges that 
+	 * We currently have no support for this in IPv4. It is not
+	 * as important in IPv4, as making pools with ranges that
 	 * leave out interfaces and hosts is fairly straightforward
 	 * using range notation, but not so handy with CIDR notation.
 	 */
@@ -844,14 +969,6 @@ main(int argc, char **argv) {
 #endif /* DHCPv6 */
 
 #ifndef DEBUG
-	if (daemon) {
-		/* First part of becoming a daemon... */
-		if ((pid = fork ()) < 0)
-			log_fatal ("Can't fork daemon: %m");
-		else if (pid)
-			exit (0);
-	}
- 
 	/*
 	 * Second part of dealing with pid files.  Now
 	 * that we have forked we can write our pid if
@@ -877,7 +994,7 @@ main(int argc, char **argv) {
 			log_fatal ("setgroups: %m");
 		if (setgid (set_gid))
 			log_fatal ("setgid(%d): %m", (int) set_gid);
-	}	
+	}
 
 	if (set_uid) {
 		if (setuid (set_uid))
@@ -893,6 +1010,15 @@ main(int argc, char **argv) {
 		log_perror = 0;
 
 	if (daemon) {
+		if (dfd[0] != -1 && dfd[1] != -1) {
+			char buf = 0;
+
+			if (write(dfd[1], &buf, 1) != 1)
+				log_fatal("write to parent: %m");
+			(void) close(dfd[1]);
+			dfd[0] = dfd[1] = -1;
+		}
+
 		/* Become session leader and get pid... */
 		(void) setsid();
 
@@ -1021,6 +1147,29 @@ void postconf_initialization (int quiet)
                         data_string_forget(&db, MDL);
                         path_dhcpd_pid = s;
                 }
+
+		oc = lookup_option(&server_universe, options,
+				   SV_LOCAL_ADDRESS6);
+		if (oc &&
+		    evaluate_option_cache(&db, NULL, NULL, NULL, options, NULL,
+					  &global_scope, oc, MDL)) {
+			if (db.len == 16) {
+				memcpy(&local_address6, db.data, 16);
+			} else
+				log_fatal("invalid local address "
+					  "data length");
+			data_string_forget(&db, MDL);
+		}
+
+		oc = lookup_option(&server_universe, options,
+				   SV_BIND_LOCAL_ADDRESS6);
+		if (oc &&
+		    evaluate_boolean_option_cache(NULL, NULL, NULL,
+						  NULL, options, NULL,
+						  &global_scope, oc, MDL)) {
+			bind_local_address6 = 1;
+		}
+
         }
 #endif /* DHCPv6 */
 
@@ -1141,9 +1290,21 @@ void postconf_initialization (int quiet)
 		}
 	}
 
-	if (dhcp_context_create(DHCP_CONTEXT_POST_DB, local4_ptr, local6_ptr)
-	    != ISC_R_SUCCESS)
-		log_fatal("Unable to complete ddns initialization");
+	/* Don't init DNS client if update style is none. This avoids
+	 * listening ports that aren't needed.  We don't use ddns-udpates
+	 * as that has multiple levels of scope. */
+	if (ddns_update_style != DDNS_UPDATE_STYLE_NONE) {
+		if (dhcp_context_create(DHCP_CONTEXT_POST_DB,
+					local4_ptr, local6_ptr)
+			!= ISC_R_SUCCESS) {
+			log_fatal("Unable to complete ddns initialization");
+		}
+	}
+
+	/* Set the conflict detection flag mask based on globally
+	 * defined DDNS configuration params.  This mask should be
+	 * to init ddns_cb::flags before for every DDNS transaction. */
+	ddns_conflict_mask = get_conflict_mask(options);
 
 #else
 	/* If we don't have support for updates compiled in tell the user */
@@ -1194,6 +1355,17 @@ void postconf_initialization (int quiet)
 		}
 		data_string_forget(&db, MDL);
 	}
+#if defined(DHCP4o6)
+	/* Delayed acks and DHCPv4-over-DHCPv6 are incompatible */
+	if (dhcpv4_over_dhcpv6) {
+		if (max_outstanding_acks > 0) {
+			log_debug("DHCP4o6 enabled, "
+				  "setting delayed-ack to zero (incompatible)");
+		}
+
+		max_outstanding_acks = 0;
+	}
+#endif
 
 	oc = lookup_option(&server_universe, options, SV_MAX_ACK_DELAY);
 	if (oc &&
@@ -1228,8 +1400,9 @@ void postconf_initialization (int quiet)
 		server_id_check = 1;
 	}
 
+#ifdef DHCPv6
 	oc = lookup_option(&server_universe, options, SV_PREFIX_LEN_MODE);
-	if ((oc != NULL) && 
+	if ((oc != NULL) &&
 	    evaluate_option_cache(&db, NULL, NULL, NULL, options, NULL,
 					  &global_scope, oc, MDL)) {
 		if (db.len == 1) {
@@ -1240,6 +1413,55 @@ void postconf_initialization (int quiet)
 
 		data_string_forget(&db, MDL);
 	}
+#endif
+
+	// Set global abandon-lease-time option.
+	oc = lookup_option (&server_universe, options, SV_ABANDON_LEASE_TIME);
+	if ((oc != NULL) &&
+	    evaluate_option_cache(&db, NULL, NULL, NULL, options, NULL,
+				  &global_scope, oc, MDL)) {
+		if (db.len == sizeof (u_int32_t)) {
+			abandon_lease_time = getULong (db.data);
+		} else {
+			log_fatal("invalid abandon-lease-time");
+		}
+
+		data_string_forget (&db, MDL);
+        }
+
+#if defined (FAILOVER_PROTOCOL)
+       oc = lookup_option(&server_universe, options, SV_CHECK_SECS_BYTE_ORDER);
+       if ((oc != NULL) &&
+	   evaluate_boolean_option_cache(NULL, NULL, NULL, NULL, options, NULL,
+					 &global_scope, oc, MDL)) {
+		check_secs_byte_order = 1;
+	}
+#endif
+
+#ifdef EUI_64
+       oc = lookup_option(&server_universe, options, SV_PERSIST_EUI_64_LEASES);
+       if (oc != NULL) {
+		persist_eui64 = evaluate_boolean_option_cache(NULL, NULL, NULL,
+							      NULL, options,
+							      NULL,
+							      &global_scope,
+							      oc, MDL);
+	}
+
+	if (!persist_eui64) {
+		log_info("EUI64 leases will not be written to lease file");
+	}
+#endif
+
+#ifdef DHCPv6
+	oc = lookup_option(&server_universe, options, SV_RELEASE_ON_ROAM);
+	if (oc != NULL) {
+		do_release_on_roam =
+			evaluate_boolean_option_cache(NULL, NULL, NULL, NULL,
+						      options, NULL,
+						      &global_scope, oc, MDL);
+	}
+#endif
 
 #if defined (BINARY_LEASES)
 	if (local_family == AF_INET) {
@@ -1372,7 +1594,7 @@ int dhcpd_interface_setup_hook (struct interface_info *ip, struct iaddr *ia)
 			interface_reference (&subnet -> interface, ip, MDL);
 			subnet -> interface_address = *ia;
 		} else if (subnet -> interface != ip) {
-			log_error ("Multiple interfaces match the %s: %s %s", 
+			log_error ("Multiple interfaces match the %s: %s %s",
 				   "same subnet",
 				   subnet -> interface -> name, ip -> name);
 		}
@@ -1386,11 +1608,11 @@ int dhcpd_interface_setup_hook (struct interface_info *ip, struct iaddr *ia)
 				shared_network_reference
 					(&ip -> shared_network, share, MDL);
 		}
-		
+
 		if (!share -> interface) {
 			interface_reference (&share -> interface, ip, MDL);
 		} else if (share -> interface != ip) {
-			log_error ("Multiple interfaces match the %s: %s %s", 
+			log_error ("Multiple interfaces match the %s: %s %s",
 				   "same shared network",
 				   share -> interface -> name, ip -> name);
 		}
@@ -1511,13 +1733,13 @@ static isc_result_t dhcp_io_shutdown_countdown (void *vlp)
 	    if (no_pid_file == ISC_FALSE)
 		    (void) unlink(path_dhcpd_pid);
 	    exit (0);
-	}		
+	}
 #else
 	if (shutdown_state == shutdown_done) {
 #if defined (DEBUG_MEMORY_LEAKAGE) && \
 		defined (DEBUG_MEMORY_LEAKAGE_ON_EXIT)
 		free_everything ();
-		omapi_print_dmalloc_usage_by_caller (); 
+		omapi_print_dmalloc_usage_by_caller ();
 #endif
 		if (no_pid_file == ISC_FALSE)
 			(void) unlink(path_dhcpd_pid);
@@ -1561,7 +1783,7 @@ isc_result_t dhcp_set_control_state (control_object_state_t oldstate,
 	/* Called on signal. */
 	log_info("Received signal %d, initiating shutdown.", shutdown_signal);
 	shutdown_signal = SIGUSR1;
-	
+
 	/*
 	 * Prompt the shutdown event onto the timer queue
 	 * and return to the dispatch loop.
