@@ -3,13 +3,12 @@
    OMAPI object interfaces for the DHCP server. */
 
 /*
- * Copyright (c) 2012-2014 by Internet Systems Consortium, Inc. ("ISC")
- * Copyright (c) 2004-2009 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2017 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1999-2003 by Internet Software Consortium
  *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
  * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
  * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
@@ -41,6 +40,9 @@ static isc_result_t class_lookup (omapi_object_t **,
 				  omapi_object_t *, omapi_object_t *,
 				  omapi_object_type_t *);
 
+static isc_result_t update_lease_flags(struct lease* lease,
+				       omapi_typed_data_t *value);
+
 omapi_object_type_t *dhcp_type_lease;
 omapi_object_type_t *dhcp_type_pool;
 omapi_object_type_t *dhcp_type_class;
@@ -227,7 +229,7 @@ isc_result_t dhcp_lease_set_value  (omapi_object_t *h,
 
 	    if (lease -> binding_state != bar) {
 		lease -> next_binding_state = bar;
-		if (supersede_lease (lease, 0, 1, 1, 1)) {
+		if (supersede_lease (lease, NULL, 1, 1, 1, 0)) {
 			log_info ("lease %s state changed from %s to %s",
 				  piaddr(lease->ip_addr), ols, nls);
 			return ISC_R_SUCCESS;
@@ -260,7 +262,7 @@ isc_result_t dhcp_lease_set_value  (omapi_object_t *h,
 		return status;
 	    old_lease_end = lease->ends;
 	    lease->ends = lease_end;
-	    if (supersede_lease (lease, 0, 1, 1, 1)) {
+	    if (supersede_lease (lease, NULL, 1, 1, 1, 0)) {
 		log_info ("lease %s end changed from %lu to %lu",
 			  piaddr(lease->ip_addr), old_lease_end, lease_end);
 		return ISC_R_SUCCESS;
@@ -269,22 +271,7 @@ isc_result_t dhcp_lease_set_value  (omapi_object_t *h,
 		      piaddr(lease->ip_addr), old_lease_end, lease_end);
 	    return ISC_R_IOERROR;
 	} else if (!omapi_ds_strcmp(name, "flags")) {
-	    u_int8_t oldflags;
-
-	    if (value->type != omapi_datatype_data)
-		return DHCP_R_INVALIDARG;
-
-	    oldflags = lease->flags;
-	    lease->flags = (value->u.buffer.value[0] & EPHEMERAL_FLAGS) |
-			   (lease->flags & ~EPHEMERAL_FLAGS);
-	    if(oldflags == lease->flags)
-		return ISC_R_SUCCESS;
-	    if (!supersede_lease(lease, NULL, 1, 1, 1)) {
-		log_error("Failed to update flags for lease %s.",
-			  piaddr(lease->ip_addr));
-		return ISC_R_IOERROR;
-	    }
-	    return ISC_R_SUCCESS;
+	    return (update_lease_flags(lease, value));
 	} else if (!omapi_ds_strcmp (name, "billing-class")) {
 	    return DHCP_R_UNCHANGED;	/* XXX carefully allow change. */
 	} else if (!omapi_ds_strcmp (name, "hardware-address")) {
@@ -321,6 +308,85 @@ isc_result_t dhcp_lease_set_value  (omapi_object_t *h,
 	return ISC_R_IOERROR;
 }
 
+/*
+ * \brief Updates the lease's flags to a given value
+ *
+ * In order to update the lease's flags, we make a copy of the
+ * lease, and update the copy's flags with the new value.
+ * We then use the updated copy as the second parameter to a
+ * call to supersede_lease().  This ensures that the lease
+ * moves between queues correctly.   This is critical when
+ * the RESERVED_LEASE flag is being changed.
+ *
+ * Note that only the EPHEMERAL flags are permitted to be changed.
+ *
+ * \param lease - pointer to the lease to update
+ * \param value - omapi data value containing the new flags value
+ *
+ * \return ISC_R_SUCCESS if the lease was successfully updated,
+ *  DHCP_R_UNCHANGED if new value would result in no change to the
+ *  lease's flags, or an appropriate status on other errors
+ */
+static isc_result_t update_lease_flags(struct lease* lease,
+				       omapi_typed_data_t *value)
+{
+	u_int8_t oldflags;
+	u_int8_t newflags;
+	struct lease* lupdate = NULL;
+	isc_result_t status;
+
+	/* Grab the requested flags value. We (the server) send flags
+	 * out as 1-byte, so we expect clients to do the same.  However
+	 * omshell, will send a network-ordered 4 byte integer if the
+	 * input is "set flags = <n>", so we'll accomdate that too. */
+	if (value->u.buffer.len == 1) {
+		newflags = value->u.buffer.value[0];
+	} else {
+		unsigned long tmp;
+
+		status = omapi_get_int_value (&tmp, value);
+		if (status != ISC_R_SUCCESS) {
+			return (status);
+		}
+
+		newflags = (u_int8_t)tmp;
+	}
+
+	/* Save off the current flags value. */
+	oldflags = lease->flags;
+
+	/* The new value must preserve all PERSISTANT_FLAGS */
+	newflags = ((lease->flags & ~EPHEMERAL_FLAGS) |
+		    (newflags & EPHEMERAL_FLAGS));
+
+	/* If there's no net change, we're done */
+	if (oldflags == newflags) {
+		return (DHCP_R_UNCHANGED);
+	}
+
+	/* Make a copy of the lease. */
+	if (!lease_copy(&lupdate, lease, MDL)) {
+		return (ISC_R_FAILURE);
+	}
+
+	/* Set the copy's flags to the new value */
+	lupdate->flags = newflags;
+
+	/* Attempt to update the lease */
+	if (!supersede_lease(lease, lupdate, 1, 1, 1, 0)) {
+		log_error("Failed to update flags for lease %s.",
+			  piaddr(lease->ip_addr));
+		status = ISC_R_FAILURE;
+	} else {
+		log_debug ("lease flags changed from  %x to %x for lease %s.",
+			   oldflags, newflags, piaddr(lease->ip_addr));
+		status = ISC_R_SUCCESS;
+	}
+
+	lease_dereference(&lupdate, MDL);
+	return (status);
+}
+
 
 isc_result_t dhcp_lease_get_value (omapi_object_t *h, omapi_object_t *id,
 				   omapi_data_string_t *name,
@@ -455,19 +521,19 @@ isc_result_t dhcp_lease_destroy (omapi_object_t *h, const char *file, int line)
 		class_dereference
 			(&lease->billing_class, file, line);
 
-#if defined (DEBUG_MEMORY_LEAKAGE) || \
-		defined (DEBUG_MEMORY_LEAKAGE_ON_EXIT)
-	/* XXX we should never be destroying a lease with a next
-	   XXX pointer except on exit... */
+	/* We no longer check for a next pointer as that should
+	 * be cleared when we destroy the pool and as before we
+	 * should only ever be doing that on exit.
 	if (lease->next)
 		lease_dereference (&lease->next, file, line);
+	 */
+
 	if (lease->n_hw)
 		lease_dereference (&lease->n_hw, file, line);
 	if (lease->n_uid)
 		lease_dereference (&lease->n_uid, file, line);
 	if (lease->next_pending)
 		lease_dereference (&lease->next_pending, file, line);
-#endif
 
 	return ISC_R_SUCCESS;
 }
@@ -1127,8 +1193,6 @@ isc_result_t dhcp_host_destroy (omapi_object_t *h, const char *file, int line)
 	if (h -> type != dhcp_type_host)
 		return DHCP_R_INVALIDARG;
 
-#if defined (DEBUG_MEMORY_LEAKAGE) || \
-		defined (DEBUG_MEMORY_LEAKAGE_ON_EXIT)
 	struct host_decl *host = (struct host_decl *)h;
 	if (host -> n_ipaddr)
 		host_dereference (&host -> n_ipaddr, file, line);
@@ -1147,7 +1211,6 @@ isc_result_t dhcp_host_destroy (omapi_object_t *h, const char *file, int line)
 		omapi_object_dereference ((omapi_object_t **)
 					  &host -> named_group, file, line);
 	data_string_forget (&host -> auth_key_id, file, line);
-#endif
 
 	return ISC_R_SUCCESS;
 }
@@ -1201,8 +1264,8 @@ isc_result_t dhcp_host_signal_handler (omapi_object_t *h,
 }
 
 isc_result_t dhcp_host_stuff_values (omapi_object_t *c,
-				      omapi_object_t *id,
-				      omapi_object_t *h)
+				     omapi_object_t *id,
+				     omapi_object_t *h)
 {
 	struct host_decl *host;
 	isc_result_t status;
@@ -1223,16 +1286,27 @@ isc_result_t dhcp_host_stuff_values (omapi_object_t *c,
 				   (struct option_state *)0,
 				   &global_scope,
 				   host -> fixed_addr, MDL)) {
+
 		status = omapi_connection_put_name (c, "ip-address");
-		if (status != ISC_R_SUCCESS)
+		if (status != ISC_R_SUCCESS) {
+			data_string_forget (&ip_addrs, MDL);
 			return status;
+		}
+
 		status = omapi_connection_put_uint32 (c, ip_addrs.len);
-		if (status != ISC_R_SUCCESS)
+		if (status != ISC_R_SUCCESS) {
+			data_string_forget (&ip_addrs, MDL);
 			return status;
+		}
+
 		status = omapi_connection_copyin (c,
 						  ip_addrs.data, ip_addrs.len);
-		if (status != ISC_R_SUCCESS)
+		if (status != ISC_R_SUCCESS) { 
+			data_string_forget (&ip_addrs, MDL);
 			return status;
+		}
+
+		data_string_forget (&ip_addrs, MDL);
 	}
 
 	if (host -> client_identifier.len) {
@@ -1579,16 +1653,11 @@ isc_result_t dhcp_pool_get_value (omapi_object_t *h, omapi_object_t *id,
 
 isc_result_t dhcp_pool_destroy (omapi_object_t *h, const char *file, int line)
 {
-#if defined (DEBUG_MEMORY_LEAKAGE) || \
-		defined (DEBUG_MEMORY_LEAKAGE_ON_EXIT)
 	struct permit *pc, *pn;
-#endif
 
 	if (h -> type != dhcp_type_pool)
 		return DHCP_R_INVALIDARG;
 
-#if defined (DEBUG_MEMORY_LEAKAGE) || \
-		defined (DEBUG_MEMORY_LEAKAGE_ON_EXIT)
 	struct pool *pool = (struct pool *)h;
 	if (pool -> next)
 		pool_dereference (&pool -> next, file, line);
@@ -1596,21 +1665,20 @@ isc_result_t dhcp_pool_destroy (omapi_object_t *h, const char *file, int line)
 		group_dereference (&pool -> group, file, line);
 	if (pool -> shared_network)
 	    shared_network_dereference (&pool -> shared_network, file, line);
-	if (pool -> active)
-		lease_dereference (&pool -> active, file, line);
-	if (pool -> expired)
-		lease_dereference (&pool -> expired, file, line);
-	if (pool -> free)
-		lease_dereference (&pool -> free, file, line);
-	if (pool -> backup)
-		lease_dereference (&pool -> backup, file, line);
-	if (pool -> abandoned)
-		lease_dereference (&pool -> abandoned, file, line);
+
+	POOL_DESTROYP(&pool->active);
+	POOL_DESTROYP(&pool->expired);
+	POOL_DESTROYP(&pool->free);
+	POOL_DESTROYP(&pool->backup);
+	POOL_DESTROYP(&pool->abandoned);
+	POOL_DESTROYP(&pool->reserved);
+
 #if defined (FAILOVER_PROTOCOL)
 	if (pool -> failover_peer)
 		dhcp_failover_state_dereference (&pool -> failover_peer,
 						 file, line);
 #endif
+
 	for (pc = pool -> permit_list; pc; pc = pn) {
 		pn = pc -> next;
 		free_permit (pc, file, line);
@@ -1622,7 +1690,6 @@ isc_result_t dhcp_pool_destroy (omapi_object_t *h, const char *file, int line)
 		free_permit (pc, file, line);
 	}
 	pool -> prohibit_list = (struct permit *)0;
-#endif
 
 	return ISC_R_SUCCESS;
 }
@@ -1733,21 +1800,14 @@ class_set_value (omapi_object_t *h,
 	class = (struct class *)h;
 
 	if (!omapi_ds_strcmp(name, "name")) {
-		char *tname;
-
 		if (class->name)
 			return ISC_R_EXISTS;
 
-		if ((tname = dmalloc(value->u.buffer.len + 1, MDL)) == NULL) {
-			return ISC_R_NOMEMORY;
-		}
-
-		/* tname is null terminated from dmalloc() */
-		memcpy(tname, value->u.buffer.value, value->u.buffer.len);
-
 		if (issubclass) {
+			char tname[value->u.buffer.len + 1];
+			memcpy(tname, value->u.buffer.value, value->u.buffer.len);
+			tname[sizeof(tname)-1] = '\0';
 			status = find_class(&superclass, tname, MDL);
-			dfree(tname, MDL);
 
 			if (status == ISC_R_NOTFOUND)
 				return status;
@@ -2114,6 +2174,8 @@ static isc_result_t class_lookup (omapi_object_t **lp,
 	status = omapi_get_value_str(ref, id, "name", &nv);
 	if (status == ISC_R_SUCCESS) {
 		char *name = dmalloc(nv->value->u.buffer.len + 1, MDL);
+		if (name == NULL)
+			return (ISC_R_NOMEMORY);
 		memcpy (name,
 			nv->value->u.buffer.value,
 			nv->value->u.buffer.len);
@@ -2197,7 +2259,9 @@ isc_result_t dhcp_class_create (omapi_object_t **lp,
 	if (status != ISC_R_SUCCESS)
 		return (status);
 
-	clone_group(&cp->group, root_group, MDL);
+	if (clone_group(&cp->group, root_group, MDL) == 0)
+		return (ISC_R_NOMEMORY);
+
 	cp->flags = CLASS_DECL_DYNAMIC;
 	status = omapi_object_reference(lp, (omapi_object_t *)cp, MDL);
 	class_dereference(&cp, MDL);
